@@ -23,7 +23,6 @@ from jax.interpreters import ad
 from jax.interpreters import batching
 from jax.interpreters import partial_eval as pe
 from jax.interpreters import mlir
-from jax.interpreters import xla
 from jax.tree_util import tree_flatten, tree_unflatten
 from jax._src import ad_util
 from jax._src import source_info_util
@@ -354,11 +353,15 @@ def remat_transpose(reduce_axes, out_cts, *in_primals, jaxpr, **params):
     in_pvals = [pe.PartialVal.unknown(x.aval) if ad.is_undefined_primal(x) else
                 pe.PartialVal.known(x) for x in in_primals]
     primal_fun = lu.wrap_init(partial(core.eval_jaxpr, jaxpr, ()))
-    tangent_jaxpr, _, consts = pe.trace_to_jaxpr(primal_fun, in_pvals, False)
-    dummy_args = [ad.UndefinedPrimal(v.aval) for v in tangent_jaxpr.invars]
-    in_cts_ = ad.backward_pass(tangent_jaxpr, reduce_axes, False, consts, dummy_args,
-                               out_cts)
-    in_cts, cell.treedef = tree_flatten(in_cts_)
+    t_jaxpr, _, consts = pe.trace_to_jaxpr_nounits(primal_fun, in_pvals, False)
+    dummy_args = [ad.UndefinedPrimal(v.aval) for v in t_jaxpr.invars]
+    in_cts = ad.backward_pass(t_jaxpr, reduce_axes, False, consts, dummy_args,
+                              out_cts)
+    in_cts_ = iter(in_cts)
+    in_cts = [next(in_cts_) if ad.is_undefined_primal(x)
+              else ad_util.Zero(x.aval) for x in in_primals]
+    assert next(in_cts_, None) is None
+    in_cts, cell.treedef = tree_flatten(in_cts)
     return in_cts
 
   args, treedef = tree_flatten((in_primals, out_cts))
@@ -393,8 +396,6 @@ def name_jvp(primals, tangents, *, name):
   return name_p.bind(x, name=name), xdot  # don't name the tangent value
 ad.primitive_jvps[name_p] = name_jvp
 
-xla.register_translation(name_p,
-                         lambda ctx, avals_in, avals_out, x, *, name: [x])
 mlir.register_lowering(name_p, lambda ctx, x, *, name: [x])
 
 def name_batcher(args, dims, *, name):
