@@ -15,6 +15,7 @@
 # Primitive dispatch and jit dispatch.
 from __future__ import annotations
 
+import atexit
 import contextlib
 from functools import partial
 import itertools
@@ -24,6 +25,7 @@ from typing import (
 from typing_extensions import Protocol
 import os
 import re
+import threading
 import warnings
 
 from absl import logging
@@ -100,6 +102,37 @@ def apply_primitive(prim, *args, **params):
 # TODO(phawkins): update code referring to xla.apply_primitive to point here.
 xla.apply_primitive = apply_primitive
 
+RuntimeToken = Any
+
+class RuntimeTokenSet(threading.local):
+  tokens: Dict[core.Effect, Tuple[RuntimeToken, Device]]
+
+  def __init__(self):
+    self.tokens = {}
+
+  def get_token(self, eff: core.Effect, device: Device) -> RuntimeToken:
+    if eff not in self.tokens:
+      self.tokens[eff] = device_put(np.zeros(0, np.bool_), device), device
+    elif self.tokens[eff][1] != device:
+      (old_token,), _ = self.tokens[eff]
+      old_token.aval = core.ShapedArray((0,), np.bool_)
+      self.tokens[eff] = device_put(old_token, device), device
+    return self.tokens[eff][0]
+
+  def update_token(self, eff: core.Effect, token: RuntimeToken):
+    self.tokens[eff] = token, self.tokens[eff][1]
+
+  def clear(self):
+    self.tokens = {}
+
+  def block_until_ready(self):
+    [t[0].block_until_ready() for t, _ in self.tokens.values()]
+
+runtime_tokens: RuntimeTokenSet = RuntimeTokenSet()
+
+@atexit.register
+def wait_for_tokens():
+  runtime_tokens.block_until_ready()
 
 @util.cache()
 def xla_primitive_callable(prim, *arg_specs: ArgSpec, **params):
@@ -113,7 +146,7 @@ def xla_primitive_callable(prim, *arg_specs: ArgSpec, **params):
     else:
       return out,
   compiled = _xla_callable_uncached(lu.wrap_init(prim_fun), device, None,
-                                    prim.name, donated_invars, *arg_specs)
+                                    prim.name, donated_invars, False, *arg_specs)
   if not prim.multiple_results:
     return lambda *args, **kw: compiled(*args, **kw)[0]
   else:
@@ -141,13 +174,13 @@ def _device_from_arg_devices(devices: Sequence[Optional[Device]]) -> Optional[De
 # JIT execution
 
 def _xla_call_impl(fun: lu.WrappedFun, *args, device, backend, name,
-                   donated_invars, inline):
+                   donated_invars, inline, keep_unused: bool):
   del inline  # Only used at tracing time
   arg_specs = unsafe_map(arg_spec, args)
   if fun.in_type is not None:
     arg_specs = [(None, *xs) for _, *xs in arg_specs]
   compiled_fun = _xla_callable(fun, device, backend, name, donated_invars,
-                               *arg_specs)
+                               keep_unused, *arg_specs)
   try:
     return compiled_fun(*args)
   except FloatingPointError:
@@ -193,9 +226,9 @@ xla.xla_call_p.def_impl(_xla_call_impl)
 
 
 def _xla_callable_uncached(fun: lu.WrappedFun, device, backend, name,
-                           donated_invars, *arg_specs):
+                           donated_invars, keep_unused, *arg_specs):
   return lower_xla_callable(fun, device, backend, name, donated_invars, False,
-                            *arg_specs).compile().unsafe_call
+                            keep_unused, *arg_specs).compile().unsafe_call
 
 _xla_callable = lu.cache(_xla_callable_uncached)
 
@@ -214,7 +247,18 @@ def log_elapsed_time(fmt: str):
 
 @profiler.annotate_function
 def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
-                       donated_invars, always_lower: bool, *arg_specs):
+                       donated_invars, always_lower: bool, keep_unused: bool,
+                       *arg_specs):
+  """Lower into XLA.
+
+  Args:
+    always_lower: If `True`, even trivial programs (not doing any computation
+      such as lambda x: x) will be lowered into an XLA program.
+    keep_unused: If `False` (the default), arguments that JAX determines to be
+      unused by `fun` *may* be dropped from resulting compiled XLA executables.
+      Such arguments will not be transferred to the device nor provided to the
+      underlying executable. If `True`, unused arguments will not be pruned.
+  """
   if device is not None and backend is not None:
     raise ValueError("can't specify both a device and a backend for jit, "
                      "got device={} and backend={}".format(device, backend))
@@ -230,7 +274,7 @@ def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
   if any(isinstance(c, core.Tracer) for c in consts):
     raise UnexpectedTracerError("Encountered an unexpected tracer.")
   # TODO(mattjj): handle argument pruning w/ dynamic shapes
-  if fun.in_type is None:
+  if fun.in_type is None and not keep_unused:
     jaxpr, kept_const_idx, kept_var_idx = _prune_unused_inputs(jaxpr)
     consts = [c for i, c in enumerate(consts) if i in kept_const_idx]
     abstract_args, arg_devices = util.unzip2(
@@ -256,7 +300,8 @@ def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
   if not jaxpr.eqns and not always_lower:
     return XlaComputation(
         name, None, True, None, None, jaxpr=jaxpr, consts=consts, device=device,
-        in_avals=abstract_args, out_avals=out_avals, kept_var_idx=kept_var_idx)
+        in_avals=abstract_args, out_avals=out_avals, effects=jaxpr.effects,
+        kept_var_idx=kept_var_idx, keepalive=None)
 
   if not _on_exit:
     log_priority = logging.WARNING if config.jax_log_compiles else logging.DEBUG
@@ -291,13 +336,15 @@ def lower_xla_callable(fun: lu.WrappedFun, device, backend, name,
   name_stack = util.new_name_stack(util.wrap_name(name, 'jit'))
   closed_jaxpr = core.ClosedJaxpr(jaxpr, consts)
   module_name = f"jit_{fun.__name__}"
-  module = mlir.lower_jaxpr_to_module(
-      module_name, closed_jaxpr, backend.platform,
+  effects = [eff for eff in closed_jaxpr.effects if eff in core.ordered_effects]
+  module, keepalive = mlir.lower_jaxpr_to_module(
+      module_name, closed_jaxpr, effects, backend.platform,
       mlir.ReplicaAxisContext(axis_env), name_stack, donated_invars)
   return XlaComputation(
       name, module, False, donated_invars, which_explicit, nreps=nreps,
       device=device, backend=backend, tuple_args=tuple_args,
-      in_avals=abstract_args, out_avals=out_avals, kept_var_idx=kept_var_idx)
+      in_avals=abstract_args, out_avals=out_avals, effects=effects,
+      kept_var_idx=kept_var_idx, keepalive=keepalive)
 
 
 def _backend_supports_unbounded_dynamic_shapes(backend: Backend) -> bool:
@@ -424,8 +471,6 @@ def aval_to_num_buffers(aval: core.AbstractValue) -> int:
   except KeyError as err:
     raise TypeError(f"No num_buffers handler for type: {type(aval)}") from err
 
-# TODO(phawkins): use zero buffers to represent a unit.
-num_buffers_handlers[core.AbstractUnit] = lambda _: 1
 num_buffers_handlers[core.AbstractToken] = lambda _: 1
 num_buffers_handlers[core.ShapedArray] = lambda _: 1
 num_buffers_handlers[core.ConcreteArray] = lambda _: 1
@@ -521,7 +566,6 @@ def dynamic_array_result_handler(sticky_device: Optional[Device],
 result_handlers: Dict[
     Type[core.AbstractValue],
     Callable[[Optional[Device], Any], ResultHandler]] = {}
-result_handlers[core.AbstractUnit] = lambda _, __: lambda _: core.unit
 result_handlers[core.AbstractToken] = lambda _, __: lambda _: core.token
 result_handlers[core.ShapedArray] = array_result_handler
 result_handlers[core.DShapedArray] = dynamic_array_result_handler
@@ -544,27 +588,48 @@ def _check_special(name, xla_shape, buf):
     if config.jax_debug_infs and np.any(np.isinf(buf.to_py())):
       raise FloatingPointError(f"invalid value (inf) encountered in {name}")
 
+def _add_tokens(effects: List[core.Effect], device, input_bufs):
+  tokens = [runtime_tokens.get_token(eff, device) for eff in effects]
+  tokens_flat = flatten(tokens)
+  input_bufs = [*tokens_flat, *input_bufs]
+  def _remove_tokens(output_bufs):
+    token_bufs, output_bufs = util.split_list(output_bufs, [len(effects)])
+    for eff, token_buf in zip(effects, token_bufs):
+      runtime_tokens.update_token(eff, token_buf)
+    return output_bufs
+  return input_bufs, _remove_tokens
+
 
 def _execute_compiled(name: str, compiled: XlaExecutable,
                       input_handler: Optional[Callable],
                       output_buffer_counts: Optional[Sequence[int]],
-                      result_handlers, kept_var_idx, *args):
+                      result_handlers,
+                      effects: List[core.Effect],
+                      kept_var_idx, *args):
   device, = compiled.local_devices()
   args = input_handler(args) if input_handler else args
   input_bufs_flat = flatten(device_put(x, device) for i, x in enumerate(args)
                             if i in kept_var_idx)
+  if effects:
+    input_bufs_flat, token_handler = _add_tokens(effects, device, input_bufs_flat)
   out_bufs_flat = compiled.execute(input_bufs_flat)
   check_special(name, out_bufs_flat)
   if output_buffer_counts is None:
     return (result_handlers[0](*out_bufs_flat),)
   out_bufs = unflatten(out_bufs_flat, output_buffer_counts)
+  if effects:
+    out_bufs = token_handler(out_bufs)
   return tuple(h(*bs) for h, bs in unsafe_zip(result_handlers, out_bufs))
 
 
 def _execute_replicated(name: str, compiled: XlaExecutable,
                         input_handler: Optional[Callable],
                         output_buffer_counts: Optional[Sequence[int]],
-                        result_handlers, kept_var_idx, *args):
+                        result_handlers,
+                        effects: List[core.Effect],
+                        kept_var_idx, *args):
+  if effects:
+    raise NotImplementedError('Cannot execute replicated computation with effects.')
   if input_handler: raise NotImplementedError  # TODO(mattjj, dougalm)
   input_bufs = [flatten(device_put(x, device) for i, x in enumerate(args)
                         if i in kept_var_idx)
@@ -580,8 +645,8 @@ def _execute_replicated(name: str, compiled: XlaExecutable,
 
 
 def _execute_trivial(jaxpr, device: Optional[Device], consts, avals, handlers,
-                     kept_var_idx, *args):
-  env = {core.unitvar: core.unit}
+                     _: List[core.Effect], kept_var_idx, *args):
+  env: Dict[core.Var, Any]  = {}
   pruned_args = (x for i, x in enumerate(args) if i in kept_var_idx)
   map(env.setdefault, jaxpr.invars, pruned_args)
   map(env.setdefault, jaxpr.constvars, consts)
@@ -704,11 +769,15 @@ def compile_or_get_cached(backend, computation, compile_options):
 
 
 class XlaCompiledComputation(stages.Executable):
-  def __init__(self, xla_executable, in_avals, kept_var_idx, unsafe_call):
+  def __init__(self, xla_executable, in_avals, kept_var_idx, unsafe_call,
+               keepalive: Any):
     self._xla_executable = xla_executable
     self.in_avals = in_avals
     self._kept_var_idx = kept_var_idx
     self.unsafe_call = unsafe_call
+    # Only the `unsafe_call` function is cached, so to avoid the `keepalive`
+    # being garbage collected we attach it to `unsafe_call`.
+    self.unsafe_call.keepalive = keepalive
 
   @staticmethod
   def from_xla_computation(
@@ -721,7 +790,9 @@ class XlaCompiledComputation(stages.Executable):
       tuple_args: bool,
       in_avals: Sequence[core.AbstractValue],
       out_avals: Sequence[core.AbstractValue],
-      kept_var_idx: Set[int]) -> XlaCompiledComputation:
+      effects: List[core.Effect],
+      kept_var_idx: Set[int],
+      keepalive: Optional[Any]) -> XlaCompiledComputation:
     sticky_device = device
     input_handler = _input_handler(backend, explicit_args, in_avals)
     result_handlers = map(partial(aval_to_result_handler, sticky_device),
@@ -735,10 +806,15 @@ class XlaCompiledComputation(stages.Executable):
       compiled = compile_or_get_cached(backend, xla_computation, options)
     buffer_counts = (None if len(out_avals) == 1 and not config.jax_dynamic_shapes
                      else [aval_to_num_buffers(aval) for aval in out_avals])
+    if effects:
+      if buffer_counts is None:
+        buffer_counts = [1]
+      buffer_counts = ([1] * len(effects)) + buffer_counts
     execute = _execute_compiled if nreps == 1 else _execute_replicated
     unsafe_call = partial(execute, name, compiled, input_handler, buffer_counts,
-                          result_handlers, kept_var_idx)
-    return XlaCompiledComputation(compiled, in_avals, kept_var_idx, unsafe_call)
+                          result_handlers, effects, kept_var_idx)
+    return XlaCompiledComputation(compiled, in_avals, kept_var_idx, unsafe_call,
+                                  keepalive)
 
   def is_trivial(self):
     return self._xla_executable == None
@@ -751,12 +827,14 @@ class XlaCompiledComputation(stages.Executable):
     return self._xla_executable
 
   @staticmethod
-  def from_trivial_jaxpr(jaxpr, consts, device, in_avals, out_avals,
-                         kept_var_idx) -> XlaCompiledComputation:
+  def from_trivial_jaxpr(jaxpr, consts, device, in_avals, out_avals, effects,
+      kept_var_idx, keepalive: Optional[Any]) -> XlaCompiledComputation:
+    assert keepalive is None
     result_handlers = map(partial(aval_to_result_handler, device), out_avals)
     unsafe_call = partial(_execute_trivial, jaxpr, device, consts,
-                          out_avals, result_handlers, kept_var_idx)
-    return XlaCompiledComputation(None, in_avals, kept_var_idx, unsafe_call)
+                          out_avals, result_handlers, effects, kept_var_idx)
+    return XlaCompiledComputation(None, in_avals, kept_var_idx, unsafe_call,
+                                  keepalive)
 
   # -- stages.Executable protocol
 
@@ -806,11 +884,6 @@ def _device_put_array(x, device: Optional[Device]):
 def _device_put_scalar(x, device):
   return _device_put_array(dtypes.coerce_to_array(x), device)
 
-def _device_put_unit(_, device):
-  backend = xb.get_device_backend(device)
-  return (backend.buffer_from_pyval(np.zeros((), dtype=np.dtype(np.bool_)),
-                                    device),)
-
 def _device_put_token(_, device):
   backend = xb.get_device_backend(device)
   return (backend.buffer_from_pyval(np.zeros((), dtype=np.dtype(np.bool_)),
@@ -821,7 +894,6 @@ _scalar_types = dtypes.python_scalar_dtypes.keys()
 device_put_handlers: Dict[Any, Callable[[Any, Optional[Device]], Tuple[Any]]] = {}
 device_put_handlers.update((t, _device_put_array) for t in array_types)
 device_put_handlers.update((t, _device_put_scalar) for t in _scalar_types)
-device_put_handlers[core.Unit] = _device_put_unit
 device_put_handlers[core.Token] = _device_put_token
 
 
