@@ -950,17 +950,12 @@ def _lu_blocked(a, block_size=128):
 
 def _lu_python(x):
   """Default LU decomposition in Python, where no better version exists."""
-  m, n = x.shape[-2:]
   batch_dims = x.shape[:-2]
-  if len(batch_dims) > 0:
-    batch_size = np.prod(batch_dims, dtype=np.int64)
-    lu, pivot, perm = api.vmap(_lu_blocked)(lax.reshape(x, (batch_size, m, n)))
-    lu = lax.reshape(lu, batch_dims + (m, n))
-    pivot = lax.reshape(pivot, batch_dims + (min(m, n),))
-    perm = lax.reshape(perm, batch_dims + (m,))
-  else:
-    lu, pivot, perm = _lu_blocked(x)
-  return lu, pivot, perm
+  fn = _lu_blocked
+  for _ in range(len(batch_dims)):
+    fn = api.vmap(fn)
+
+  return fn(x)
 
 def _lu_impl(operand):
   lu, pivot, perm = xla.apply_primitive(lu_p, operand)
@@ -1612,18 +1607,19 @@ def _schur_cpu_lowering(ctx, operand, *, compute_schur_vectors, sort_eig_vals,
   operand_aval, = ctx.avals_in
   batch_dims = operand_aval.shape[:-2]
 
-  if sort_eig_vals:
-    T, vs, _sdim, info = lapack.gees_mhlo(
-        operand,
-        jobvs=compute_schur_vectors,
-        sort=sort_eig_vals,
-        select=select_callable)
-  else:
-    T, vs, info = lapack.gees_mhlo(
-        operand,
-        jobvs=compute_schur_vectors,
-        sort=sort_eig_vals,
-        select=select_callable)
+  # TODO(jakevdp): remove this try/except when minimum jaxlib >= 0.3.8
+  try:
+    gees_result = lapack.gees_mhlo(operand_aval.dtype, operand,
+                                   jobvs=compute_schur_vectors,
+                                   sort=sort_eig_vals,
+                                   select=select_callable)
+  except TypeError:  # API for jaxlib <= 0.3.7
+    gees_result = lapack.gees_mhlo(operand,  # pytype: disable=missing-parameter
+                                   jobvs=compute_schur_vectors,
+                                   sort=sort_eig_vals,
+                                   select=select_callable)
+  # Number of return values depends on value of sort_eig_vals.
+  T, vs, *_, info = gees_result
 
   ok = mlir.compare_mhlo(
       info, mlir.full_like_aval(0, ShapedArray(batch_dims, np.dtype(np.int32))),
