@@ -34,6 +34,20 @@ if platform.system() == "Darwin" and platform.machine() == "arm64":
                 "Please see https://github.com/google/jax/issues/5501 in the "
                 "event of problems.")
 
+
+# This apparently-unused import is to work around
+# https://github.com/google/jax/issues/9218#issuecomment-1016949739
+# If the user is using Conda, we want to ensure that Conda's libstdc++ is chosen
+# by the dynamic linker in preference to the system libstdc++. Before importing
+# jaxlib, we first must import a library that:
+# a) is likely to be provided by Conda and not hand-installed by the user, and
+# b) uses C++ and links to libstdc++.
+# Some submodules of scipy (e.g., scipy.signal) satisfy this criterion.
+# If the user isn't using Conda, this code merely wastes a bit of time, but
+# we're likely to import scipy anyway later.
+import scipy.signal as _signal
+del _signal
+
 try:
   import jaxlib as jaxlib
 except ModuleNotFoundError as err:
@@ -159,7 +173,7 @@ xla_extension_version = getattr(xla_client, '_version', 0)
 _xla_extension_version = xla_extension_version
 
 # Version number for MLIR:Python APIs, provided by jaxlib.
-mlir_api_version = getattr(xla_client, 'mlir_api_version', 0)
+mlir_api_version = xla_client.mlir_api_version
 
 try:
   from jaxlib import tpu_client as tpu_driver_client  # pytype: disable=import-error
@@ -173,5 +187,4 @@ cuda_path = os.path.join(os.path.dirname(jaxlib.__file__), "cuda")
 if not os.path.isdir(cuda_path):
   cuda_path = None
 
-if xla_extension_version >= 58:
-  transfer_guard_lib = xla_client._xla.transfer_guard_lib
+transfer_guard_lib = xla_client._xla.transfer_guard_lib
