@@ -18,7 +18,7 @@ import types
 from typing import Any, Callable, TypeVar
 
 import jax
-from jax._src.lib import xla_extension, xla_extension_version
+from jax._src.lib import xla_extension
 from jax._src import util
 
 C = TypeVar("C", bound=Callable[..., Any])
@@ -71,9 +71,6 @@ def filter_traceback(tb):
   for f, lineno in reversed(frames):
     if include_frame(f):
       out = types.TracebackType(out, f, f.f_lasti, lineno)  # pytype: disable=wrong-arg-count
-  if xla_extension_version < 54 and out is None and len(frames) > 0:
-    f, lineno = frames[-1]
-    out = types.TracebackType(out, f, f.f_lasti, lineno)
   return out
 
 def add_call_stack_frames(tb):
@@ -175,8 +172,6 @@ def api_boundary(fun: C) -> C:
       filtered_tb, unfiltered, mode = None, None, None
       try:
         filtered_tb = filter_traceback(e.__traceback__)
-        if xla_extension_version < 54 and filtered_tb is None:
-          raise
         msg = format_exception_only(e)
         msg = f'{msg}\n\n{_jax_message_append}'
         unfiltered = UnfilteredStackTrace(msg)
