@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from functools import partial
 import itertools as it
 
 from typing import Any, Callable, Dict, Sequence, Union
@@ -24,11 +25,11 @@ from jax import lax
 from jax import custom_derivatives as cd
 from jax.interpreters import partial_eval as pe
 from jax import linear_util as lu
-from jax._src.util import partial, safe_map, wraps, split_list
+from jax._src.util import safe_map, wraps, split_list
 from jax._src.lax import control_flow as lcf
 
 import inspect
-from jax.api_util import flatten_fun_nokwargs
+from jax._src.api_util import flatten_fun_nokwargs
 from jax.tree_util import tree_flatten, tree_unflatten, tree_structure, tree_map
 
 map = safe_map
