@@ -717,7 +717,7 @@ def make_xmap_callable(fun: lu.WrappedFun,
         tiling_method=tiling_method, in_is_global=in_is_global)
   else:
     return dispatch.lower_xla_callable(
-        f, None, backend, name, donated_invars, False,
+        f, None, backend, name, donated_invars, False, True,
         *[(a, None) for a in in_avals])
 
 class EvaluationPlan(NamedTuple):
@@ -1106,25 +1106,6 @@ def out_local_named_shapes(local_axes, *args, **kwargs):
   ans_axes = [frozenset(a.aval.named_shape) & local_axes for a in ans]
   yield ans, ans_axes
 
-@lu.transformation_with_aux
-def hide_units(unit_args, *args, **kwargs):
-  ans = yield restore_units(unit_args, args), kwargs
-  yield filter_units(ans)
-
-def filter_units(vals):
-  vals_no_units = [v for v in vals if v is not core.unit]
-  vals_is_unit = [v is core.unit for v in vals]
-  return vals_no_units, vals_is_unit
-
-def restore_units(is_unit, vals):
-  vals_it = iter(vals)
-  vals_with_units = [core.unit if u else next(vals_it) for u in is_unit]
-  try:
-    next(vals_it)
-    raise RuntimeError("Expected the iterator to be exhausted")
-  except StopIteration:
-    return vals_with_units
-
 
 def _jaxpr_trace_process_xmap(self, primitive, f: lu.WrappedFun, tracers, params):
   assert primitive is xmap_p
@@ -1370,7 +1351,6 @@ def _xmap_lowering_rule_replica(ctx, *in_nodes,
                                    tokens_in=ctx.tokens_in,
                                    tokens_out=ctx.tokens_out),
           in_node)[0]
-    if v.aval is not core.abstract_unit else in_node
     for v, aval, in_node, arg_in_axes
     in zip(call_jaxpr.invars, ctx.avals_in, in_nodes, mesh_in_axes))
 
@@ -1397,7 +1377,6 @@ def _xmap_lowering_rule_replica(ctx, *in_nodes,
                                        avals_out=None,
                                        tokens_in=ctx.tokens_in,
                                        tokens_out=ctx.tokens_out), tiled_out)[0]
-      if v.aval is not core.abstract_unit else tiled_out
       for v, vectorized_outvar, tiled_out, ans_out_axes
       in zip(call_jaxpr.outvars, vectorized_jaxpr.outvars, tiled_outs,
              mesh_out_axes)]
