@@ -48,10 +48,9 @@ from jax.tree_util import (tree_map, tree_flatten, tree_unflatten,
 from jax._src.tree_util import prefix_errors
 from jax._src import util
 from jax._src.util import (
-    extend_name_stack, HashableFunction, safe_map, safe_zip, wrap_name, wraps,
-    distributed_debug_log, split_list, cache, tuple_insert, weakref_lru_cache)
-
-xops = xc._xla.ops
+    HashableFunction, safe_map, safe_zip, wrap_name, wraps,
+    distributed_debug_log, split_list, cache, tuple_insert, weakref_lru_cache,
+    merge_lists)
 
 class _FromGdaSingleton:
   pass
@@ -687,42 +686,6 @@ def _pjit_abstract_eval(*args, jaxpr, out_axis_resources, resource_env,
 pjit_p.def_effectful_abstract_eval(_pjit_abstract_eval)
 
 
-def _pjit_translation_rule(ctx, avals_in, avals_out, *in_nodes, name,
-                           jaxpr, in_axis_resources, out_axis_resources,
-                           resource_env, donated_invars, in_positional_semantics,
-                           out_positional_semantics):
-  # TODO: Make this into an MLIR rule and use manual_axes!
-  mesh = resource_env.physical_mesh
-  subc = xc.XlaBuilder(f"pjit_{name}")
-
-  args = []
-  for i, (n, aval, axis_resources) in enumerate(
-      safe_zip(in_nodes, avals_in, in_axis_resources)):
-    # N.B. inlined calls shouldn't have shardings set directly on the inputs or
-    # outputs (set_sharding_proto adds an identity operation).
-    arg = xla.parameter(subc, i, ctx.builder.GetShape(n))
-    args.append(
-        xla.set_sharding_proto(
-            subc, arg, get_aval_sharding_proto(aval, axis_resources, mesh)))
-
-  # TODO: Think about how to avoid duplicating constants with the outer jaxpr
-  sub_ctx = ctx.replace(
-      builder=subc,
-      name_stack=extend_name_stack(ctx.name_stack, wrap_name(name, "pjit")))
-  out_nodes = xla.jaxpr_subcomp(
-      sub_ctx, jaxpr.jaxpr, xla._xla_consts(subc, jaxpr.consts), *args)
-  out_nodes = [
-      xla.set_sharding_proto(subc, out,
-                             get_aval_sharding_proto(aval, axis_resources, mesh))
-      for out, aval, axis_resources in safe_zip(
-          out_nodes, avals_out, out_axis_resources)
-  ]
-
-  subc = subc.build(xops.Tuple(subc, out_nodes))
-  return xla.xla_destructure(ctx.builder,
-                             xops.Call(ctx.builder, subc, list(in_nodes)))
-xla.register_translation(pjit_p, _pjit_translation_rule)
-
 def _pjit_lowering(ctx, *args, name, jaxpr, in_axis_resources,
                    out_axis_resources, resource_env, donated_invars,
                    in_positional_semantics, out_positional_semantics):
@@ -750,7 +713,7 @@ def _pjit_lowering(ctx, *args, name, jaxpr, in_axis_resources,
   # TODO(b/228598865): inlined calls cannot have shardings set directly on the
   # inputs or outputs because they are lost during MHLO->HLO conversion.
   # using_sharding_annotation=False means we add an identity operation instead.
-  func = mlir.lower_jaxpr_to_fun(sub_ctx, f"pjit_{name}", jaxpr,
+  func = mlir.lower_jaxpr_to_fun(sub_ctx, f"pjit_{name}", jaxpr, (),
                                  arg_shardings=arg_shardings,
                                  result_shardings=result_shardings,
                                  use_sharding_annotations=False)
@@ -841,21 +804,15 @@ def _pjit_partial_eval(trace, *in_tracers,
 
   known_ins = tuple(pv.is_known() for pv in in_pvals)
   unknown_ins = tuple(not k for k in known_ins)
-  raw_known_jaxpr, raw_unknown_jaxpr, unknown_outs = pe.partial_eval_jaxpr(
+  known_jaxpr, unknown_jaxpr, unknown_outs, res_avals = pe.partial_eval_jaxpr_nounits(
       jaxpr, unknown_ins, instantiate=False)
   unknown_outs = tuple(unknown_outs)
   known_outs = tuple(not uk for uk in unknown_outs)
-  num_residuals = len(raw_known_jaxpr.jaxpr.outvars) - len(unknown_outs)
+  num_residuals = len(res_avals)
 
   def keep_where(l, should_keep):
     return tuple(x for x, keep in zip(l, should_keep) if keep)
 
-  # Prepare the known jaxpr
-  # TODO(apaszke): map_jaxpr will break caching!
-  known_jaxpr = raw_known_jaxpr.map_jaxpr(lambda jaxpr: pe._drop_vars(
-      jaxpr,
-      drop_ins=unknown_ins,
-      drop_outs=unknown_outs + (False,) * num_residuals))
   # Compute the known outputs
   known_params = dict(
       jaxpr=known_jaxpr,
@@ -888,18 +845,18 @@ def _pjit_partial_eval(trace, *in_tracers,
       *(pv.get_known() for pv in in_pvals if pv.is_known()),
       **known_params)
   if num_residuals:
-    known_out_vals, residual_vals = split_list(all_known_outs, [-num_residuals])
+    known_out_vals, residual_vals = \
+        split_list(all_known_outs, [len(all_known_outs) - num_residuals])
   else:
     known_out_vals, residual_vals = all_known_outs, ()
-  known_tracers_out = [trace.new_const(known_out) for known_out in known_out_vals]
   residual_tracers = [trace.new_instantiated_const(residual) for residual in residual_vals]
 
-  # Prepare the unknown jaxpr
-  # TODO(apaszke): map_jaxpr will break caching!
-  unknown_jaxpr = raw_unknown_jaxpr.map_jaxpr(lambda jaxpr: pe._drop_vars(
-      jaxpr,
-      drop_ins=known_ins + (False,) * num_residuals,
-      drop_outs=known_outs))
+  # The convention of partial_eval_jaxpr_nounits is to place residual binders
+  # at the front of the jaxpr produced, so we move them to the back since both
+  # the jaxpr equation built below and the pjit transpose rule assume a
+  # residual-inputs-last convention.
+  unknown_jaxpr = pe.move_binders_to_back(
+      unknown_jaxpr, [True] * num_residuals + [False] * sum(unknown_ins))
   # Prepare unknown tracers
   unknown_params = dict(
       jaxpr=unknown_jaxpr,
@@ -926,7 +883,7 @@ def _pjit_partial_eval(trace, *in_tracers,
                           unknown_jaxpr.effects,
                           source_info_util.current())
   for t in unknown_tracers_out: t.recipe = eqn
-  return pe._zip_knowns(known_tracers_out, unknown_tracers_out, unknown_outs)
+  return merge_lists(unknown_outs, known_out_vals, unknown_tracers_out)
 pe.custom_partial_eval_rules[pjit_p] = _pjit_partial_eval
 
 
@@ -1046,20 +1003,6 @@ ad.deflinear2(sharding_constraint_p,
                   sharding_constraint_p.bind(
                       ct, axis_resources=axis_resources, resource_env=resource_env),))
 
-def _sharding_constraint_translation_rule(ctx, avals_in, avals_out, x_node, *,
-                                          axis_resources, resource_env):
-  aval, = avals_in
-  mesh = resource_env.physical_mesh
-  return [
-      xla.set_sharding_proto(
-          ctx.builder,
-          x_node,
-          get_aval_sharding_proto(
-              aval, axis_resources, mesh, allow_uneven_axes=True),
-          unspecified_dims=get_unconstrained_dims(axis_resources))
-  ]
-xla.register_translation(sharding_constraint_p, _sharding_constraint_translation_rule)
-
 def _sharding_constraint_mhlo_lowering(ctx, x_node, *, axis_resources,
                                        resource_env):
   aval, = ctx.avals_in
