@@ -141,7 +141,6 @@ def aval_to_ir_types(aval: core.AbstractValue) -> Sequence[ir.Type]:
   except KeyError as err:
     raise TypeError(f"No ir_type_handler for aval type: {type(aval)}") from err
 
-ir_type_handlers[core.AbstractUnit] = lambda _: ()
 ir_type_handlers[core.AbstractBInt] = _bint_ir_types
 ir_type_handlers[core.ShapedArray] = _array_ir_types
 ir_type_handlers[core.ConcreteArray] = _array_ir_types
@@ -201,13 +200,13 @@ def ir_constant(val: Any, canonicalize_types: bool = True) -> ir.Value:
                     f"multiple IR values {values}")
   return values[0]
 
-register_constant_handler(core.Unit, lambda val, canonicalize_types: ())
 
 def _numpy_array_constant(x: np.ndarray, canonicalize_types
                          ) -> Sequence[ir.Value]:
   if canonicalize_types:
     x = np.asarray(x, dtypes.canonicalize_dtype(x.dtype))
-  ir_type = ir.RankedTensorType.get(x.shape, dtype_to_ir_type(x.dtype))
+  element_type = dtype_to_ir_type(x.dtype)
+  ir_type = ir.RankedTensorType.get(x.shape, element_type)
   shape = x.shape
   if x.dtype == np.bool_:
     nelems = x.size
@@ -219,8 +218,12 @@ def _numpy_array_constant(x: np.ndarray, canonicalize_types
   elif x.dtype == dtypes.bfloat16:
     x = x.view(np.uint16)
   x = np.ascontiguousarray(x)
-  attr = ir.DenseElementsAttr.get(x, type=ir_type.element_type, shape=shape)
-  return (mhlo.ConstOp(ir_type, attr).result,)
+  attr = ir.DenseElementsAttr.get(x, type=element_type, shape=shape)
+  if jax._src.lib.xla_extension_version >= 64:
+    return (mhlo.ConstOp(attr).result,)
+  else:
+    return (mhlo.ConstOp(ir_type, attr).result,)
+
 
 
 def _ndarray_constant_handler(val: np.ndarray, canonicalize_types
@@ -365,6 +368,7 @@ class ModuleContext:
   platform: str
   axis_context: AxisContext
   name_stack: NameStack
+  keepalives: List[Any]
 
   # Cached primitive lowerings.
   cached_primitive_lowerings: Dict[Any, func_dialect.FuncOp]
@@ -378,6 +382,7 @@ class ModuleContext:
       platform: str,
       axis_context: AxisContext,
       name_stack: NameStack,
+      keepalives: List[Any],
       context: Optional[ir.Context] = None,
       module: Optional[ir.Module] = None,
       ip: Optional[ir.InsertionPoint] = None,
@@ -393,6 +398,10 @@ class ModuleContext:
     self.name_stack = name_stack
     self.cached_primitive_lowerings = ({} if cached_primitive_lowerings is None
                                        else cached_primitive_lowerings)
+    self.keepalives = keepalives
+
+  def add_keepalive(self, keepalive: Any) -> None:
+    self.keepalives.append(keepalive)
 
   def replace(self, **kw): return dataclasses.replace(self, **kw)
 
@@ -473,14 +482,17 @@ def sharded_aval(aval: core.ShapedArray,
     sharded_shape.append((aval.shape[i] + partitions - 1) // partitions)
   return aval.update(tuple(sharded_shape))
 
+
 def lower_jaxpr_to_module(
-    module_name: str, jaxpr: core.ClosedJaxpr, platform: str,
+    module_name: str, jaxpr: core.ClosedJaxpr,
+    effects: List[core.Effect],
+    platform: str,
     axis_context: AxisContext,
     name_stack: NameStack, donated_args: Sequence[bool],
     replicated_args: Optional[Sequence[bool]] = None,
     arg_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None,
     result_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None
-    ) -> ir.Module:
+    ) -> Tuple[ir.Module, Optional[Any]]:
   """Lowers a top-level jaxpr to an MHLO module.
 
   Handles the quirks of the argument/return value passing conventions of the
@@ -504,6 +516,8 @@ def lower_jaxpr_to_module(
   if platform in platforms_with_donation:
     input_output_aliases, donated_args = _set_up_aliases(
         in_avals, out_avals, donated_args)
+  if any(eff not in lowerable_effects for eff in jaxpr.effects):
+    raise ValueError(f'Cannot lower jaxpr with effects: {jaxpr.effects}')
   if any(donated_args):
     # TODO(tomhennigan): At call time we should mark these buffers as deleted.
     unused_donations = [str(a) for a, d in zip(in_avals, donated_args)
@@ -513,7 +527,9 @@ def lower_jaxpr_to_module(
       msg = f"Donation is not implemented for {platform}.\n{msg}"
     warnings.warn(f"Some donated buffers were not usable: {', '.join(unused_donations)}.\n{msg}")
 
-  ctx = ModuleContext(platform, axis_context, name_stack)
+  # Create a keepalives list that will be mutated during the lowering.
+  keepalives: List[Any] = []
+  ctx = ModuleContext(platform, axis_context, name_stack, keepalives)
   with ctx.context, ir.Location.unknown(ctx.context):
     # Remove module name characters that XLA would alter. This ensures that
     # XLA computation preserves the module name.
@@ -522,22 +538,19 @@ def lower_jaxpr_to_module(
     # This may or may not be a reasonable assumption.
     ctx.module.operation.attributes["sym_name"] = ir.StringAttr.get(
         f"{module_name}.{next(_module_unique_id)}")
-    # TODO(phawkins): represent units with zero buffers at the runtime level.
     unlowerable_effects = {eff for eff in jaxpr.effects
                            if eff not in lowerable_effects}
     if unlowerable_effects:
       raise ValueError(
           f'Cannot lower jaxpr with unlowerable effects: {unlowerable_effects}')
-    effects = [eff for eff in jaxpr.effects if eff in core.ordered_effects]
     lower_jaxpr_to_fun(
         ctx, "main", jaxpr, effects, public=True, create_tokens=True,
-        replace_units_with_dummy=True,
         replace_tokens_with_dummy=True, replicated_args=replicated_args,
         arg_shardings=arg_shardings, result_shardings=result_shardings,
         input_output_aliases=input_output_aliases)
 
   ctx.module.operation.verify()
-  return ctx.module
+  return ctx.module, ctx.keepalives
 
 def module_to_string(module: ir.Module) -> str:
   output = io.StringIO()
@@ -623,6 +636,12 @@ class TokenSet:
         new_tokens.append(self._tokens[eff])
     return TokenSet(zip(self.effects(), new_tokens))
 
+def dummy_token_type() -> Sequence[ir.Type]:
+  return aval_to_ir_types(core.ShapedArray((0,), np.bool_))
+
+def dummy_token() -> Sequence[ir.Value]:
+  return ir_constants(np.zeros(0, np.bool_))
+
 def lower_jaxpr_to_fun(
     ctx: ModuleContext,
     name: str,
@@ -631,7 +650,6 @@ def lower_jaxpr_to_fun(
     *,
     create_tokens: bool = False,
     public: bool = False,
-    replace_units_with_dummy: bool = False,
     replace_tokens_with_dummy: bool = False,
     replicated_args: Optional[Sequence[bool]] = None,
     arg_shardings: Optional[Sequence[Optional[xc.OpSharding]]] = None,
@@ -650,9 +668,8 @@ def lower_jaxpr_to_fun(
     jaxpr: the jaxpr to lower.
     effects: a sequence of `core.Effect`s corresponding to an ordering of tokens
       that will be created in or used by the lowered function.
+    create_tokens: if true, the MHLO will create tokens and ignore dummy input tokens.
     public: if true, the function's visibility is set to "public".
-    replace_units_with_dummy: if true, unit arguments/return values are
-      replaced with bool arrays of size [0].
     replace_tokens_with_dummy: if true, token arguments/return values are
       replaced with bool arrays of size [0].
     replicated_args: if present, annotates arguments as replicated.
@@ -668,18 +685,16 @@ def lower_jaxpr_to_fun(
   Returns the name of the function.
   """
   def aval_to_types(aval):
-    if replace_units_with_dummy and aval is core.abstract_unit:
-      aval = core.ShapedArray((), np.dtype(np.bool_))
-    elif replace_tokens_with_dummy and aval is core.abstract_token:
+    if replace_tokens_with_dummy and aval is core.abstract_token:
       aval = core.ShapedArray((), np.dtype(np.bool_))
     return aval_to_ir_types(aval)
 
   input_types = map(aval_to_types, jaxpr.in_avals)
   output_types = map(aval_to_types, jaxpr.out_avals)
+  num_tokens = len(effects)
   if create_tokens:
     # If we create the tokens they won't be inputs to the MLIR function.
-    num_tokens = 0
-    token_types = []
+    token_types = [dummy_token_type() for _ in effects]
   else:
     # If we aren't creating tokens they will be the initial inputs to the
     # MLIR function.
@@ -776,9 +791,7 @@ def lower_jaxpr_to_fun(
       tokens_in = TokenSet(zip(effects, token_args))
     args: List[List[ir.Value]] = []
     for aval, arg in zip(jaxpr.in_avals, unflattened_args):
-      if replace_units_with_dummy and aval is core.abstract_unit:
-        args.append([])
-      elif replace_tokens_with_dummy and aval is core.abstract_token:
+      if replace_tokens_with_dummy and aval is core.abstract_token:
         args.append(mhlo.CreateTokenOp(mhlo.TokenType.get()).results)
       else:
         args.append(arg)
@@ -789,16 +802,13 @@ def lower_jaxpr_to_fun(
                                          *args)
     outs = []
     if create_tokens:
-      # If we created the tokens in this function, we are done with them and can
-      # ignore `tokens_out`.
-      pass
+      for _ in effects:
+        outs.append(dummy_token())
     else:
       for token in tokens_out.tokens():
         outs.append(token)
     for aval, out in zip(jaxpr.out_avals, out_vals):
-      if replace_units_with_dummy and aval is core.abstract_unit:
-        outs.append(ir_constants(np.zeros((), np.bool_)))
-      elif replace_tokens_with_dummy and aval is core.abstract_token:
+      if replace_tokens_with_dummy and aval is core.abstract_token:
         outs.append(ir_constants(np.zeros((), np.bool_)))
       else:
         outs.append(out)
@@ -871,7 +881,6 @@ def jaxpr_subcomp(ctx: ModuleContext, jaxpr: core.Jaxpr,
   assert len(args) == len(jaxpr.invars), (jaxpr, args)
   assert len(consts) == len(jaxpr.constvars), (jaxpr, consts)
   assert all(isinstance(v, ir.Value) for vs in consts for v in vs), consts
-  write(core.unitvar, ())
   map(write, jaxpr.constvars, consts)
   map(write, jaxpr.invars, args)
   for eqn in jaxpr.eqns:
@@ -984,8 +993,8 @@ def _call_lowering(fn_name, stack_name, call_jaxpr, backend, ctx, avals_in,
 
 def _xla_call_lower(ctx, *args,
                     backend=None, name, call_jaxpr, donated_invars, inline=None,
-                    device=None):
-  del device, donated_invars, inline  # Ignored.
+                    device=None, keep_unused=None):
+  del device, donated_invars, inline, keep_unused  # Ignored.
   out_nodes, tokens = _call_lowering(
       name, util.wrap_name(name, "jit"), call_jaxpr, backend,
       ctx.module_context, ctx.avals_in, ctx.avals_out, ctx.tokens_in, *args)
