@@ -62,9 +62,9 @@ class PartialVal(tuple):
   """Partial value: either a known value or an unknown (abstract) value.
 
   Represented as a pair `(aval_opt, const)` of one of two kinds:
-  * `(None, <Constant>)` indicates a known value, either a Python regular
-    value, or a Tracer.
-  * `(<AbstractValue>, *)` indicates an unknown value characterized by an
+  * `(None, <Constant>)` indicates a known value, where the constant is either a
+    Tracer or satisfies `core.valid_jaxtype(const)`;
+  * `(<AbstractValue>, None)` indicates an unknown value characterized by an
     abstract value.
   """
   def __new__(cls, xs: Tuple[Optional[AbstractValue], core.Value]):
@@ -72,10 +72,10 @@ class PartialVal(tuple):
     if config.jax_enable_checks:
       # type checks
       assert isinstance(pv, (AbstractValue, type(None))), xs
-      assert isinstance(const, core.Tracer) or type(const) is Zero or core.valid_jaxtype(const), xs
+      assert (const is None or isinstance(const, core.Tracer) or
+              core.valid_jaxtype(const)), const
       # invariant checks
-      if isinstance(pv, AbstractValue):
-        assert get_aval(const) == core.abstract_unit, xs
+      assert (pv is None) ^ (const is None)
     return tuple.__new__(cls, xs)
 
   @classmethod
@@ -84,7 +84,7 @@ class PartialVal(tuple):
 
   @classmethod
   def unknown(cls, aval: AbstractValue) -> PartialVal:
-    return PartialVal((aval, core.unit))
+    return PartialVal((aval, None))
 
   def is_known(self) -> bool:
     return self[0] is None
@@ -120,7 +120,7 @@ class JaxprTrace(Trace):
   def new_const(self, val) -> JaxprTracer:
     if isinstance(val, Tracer) and val._trace.level == self.level:
       raise Exception
-    return JaxprTracer(self, PartialVal.known(val), core.unit)
+    return JaxprTracer(self, PartialVal.known(val), None)
 
   def new_instantiated_literal(self, val) -> JaxprTracer:
     aval = get_aval(val)
@@ -520,7 +520,7 @@ def abstract_eval_fun(fun, *avals, debug_info=None, **params):
 
 
 JaxprTracerRecipe = Union['JaxprEqnRecipe', 'LambdaBinding', 'FreeVar',
-                          'ConstVar', Literal, core.Unit]
+                          'ConstVar', Literal]
 
 class JaxprTracer(Tracer):
   __slots__ = ['pval', 'recipe']
@@ -709,8 +709,8 @@ def tracers_to_jaxpr(
   def getvar(t: JaxprTracer) -> Atom:
     var = t_to_var.get(id(t))
     if var is None:
-      aval = t.pval.get_aval() if not t.pval.is_known() else core.abstract_unit
-      var = t_to_var[id(t)] = newvar(aval)
+      assert not t.pval.is_known()
+      var = t_to_var[id(t)] = newvar(t.pval.get_aval())
     return var
   sorted_tracers = toposort(out_tracers)
   invars = map(getvar, in_tracers)
@@ -742,8 +742,8 @@ def tracers_to_jaxpr(
       consts[v] = recipe.val
     elif isinstance(recipe, Literal):
       t_to_var[id(t)] = recipe
-    elif recipe is core.unit:
-      t_to_var[id(t)] = core.unitvar
+    elif recipe is None:
+      assert False
     else:
       raise TypeError(recipe)
 
@@ -777,10 +777,6 @@ def convert_envvars_to_constvars(jaxpr: Jaxpr, num_env_vars: int) -> Jaxpr:
   return converted_jaxpr
 
 
-def _split_aval(unknown: bool, aval: AbstractValue) -> Tuple[AbstractValue, AbstractValue]:
-  return (core.abstract_unit, aval) if unknown else (aval, core.abstract_unit)
-
-
 def partial_eval_jaxpr_nounits(
     jaxpr: ClosedJaxpr, unknowns: Sequence[bool],
     instantiate: Union[bool, Sequence[bool]],
@@ -1044,7 +1040,6 @@ def _partial_eval_jaxpr_custom(
     return x
 
   known_eqns, staged_eqns = [], []
-  write(False, True, core.unitvar)
   map(write, in_unknowns, [True] * len(in_unknowns), jaxpr.invars)
   for eqn in jaxpr.eqns:
     unks_in, inst_in = unzip2(map(read, eqn.invars))
@@ -1070,8 +1065,7 @@ def _partial_eval_jaxpr_custom(
   assert all(type(v) is Var for v in residuals), residuals
 
   ins_known, _ = partition_list(in_unknowns, jaxpr.invars)
-  outs_known_, _ = partition_list(out_unknowns, jaxpr.outvars)
-  outs_known = [x for x in outs_known_ if x.aval is not core.abstract_unit]
+  outs_known, _ = partition_list(out_unknowns, jaxpr.outvars)
   known_effects = core.join_effects(*(eqn.effects for eqn in known_eqns))
   jaxpr_known = Jaxpr((), ins_known, [*outs_known, *residuals], known_eqns,
                       known_effects)
@@ -1122,11 +1116,7 @@ def call_partial_eval_custom_rule(
   jaxpr_known, jaxpr_staged, unks_out, inst_out, num_res = \
       _partial_eval_jaxpr_custom(jaxpr, unks_in, saveable)
   ins_known, _ = partition_list(unks_in, eqn.invars)
-  # by convention, _partial_eval_jaxpr_custom drops units on known outputs
-  known_units_out = [v.aval is core.abstract_unit for v in jaxpr.outvars]
-  dropped_outs_known = map(op.or_, unks_out, known_units_out)
-  kept_outs_known = [not d for d in dropped_outs_known]
-  out_binders_known, _ = partition_list(dropped_outs_known, eqn.outvars)
+  out_binders_known, _ = partition_list(unks_out, eqn.outvars)
   _, out_binders_staged = partition_list(inst_out, eqn.outvars)
   kept_outs_staged = inst_out
   newvar = core.gensym([jaxpr_known, jaxpr_staged])
@@ -1134,7 +1124,8 @@ def call_partial_eval_custom_rule(
   params_known = {**eqn.params, jaxpr_param_name: jaxpr_known}
   params_staged = {**eqn.params, jaxpr_param_name: jaxpr_staged}
   params_known, params_staged = params_updater(
-      unks_in, kept_outs_known, kept_outs_staged, num_res, params_known, params_staged)
+      unks_in, map(op.not_, unks_out), kept_outs_staged, num_res, params_known,
+      params_staged)
   eqn_known = new_jaxpr_eqn(ins_known, [*out_binders_known, *residuals],
                             eqn.primitive, params_known, jaxpr_known.effects, eqn.source_info)
   eqn_staged = new_jaxpr_eqn([*residuals, *eqn.invars], out_binders_staged,
@@ -2085,7 +2076,6 @@ def _eval_jaxpr_padded(
   def write(v, val) -> None:
     env[v] = val
 
-  write(core.unitvar, core.unit)
   map(write, jaxpr.constvars, consts)
   map(write, jaxpr.invars, args)
   for eqn in jaxpr.eqns:
