@@ -58,11 +58,8 @@ def _avals_to_results_handler(nrep, npart, partitions, out_avals):
   return handler
 
 def _aval_to_result_handler(npart, parts, aval):
-  if aval is not core.abstract_unit:
-    spec = pxla.partitioned_sharding_spec(npart, parts, aval)
-    indices = pxla.spec_to_indices(aval.shape, spec)
-  else:
-    spec = indices = None
+  spec = pxla.partitioned_sharding_spec(npart, parts, aval)
+  indices = pxla.spec_to_indices(aval.shape, spec)
   return pxla.local_aval_to_result_handler(aval, spec, indices)
 
 
@@ -140,9 +137,11 @@ def _sharded_callable(
               fun.__name__, nparts, global_abstract_args)
 
   axis_env = xla.AxisEnv(nrep, (), ())
-  module = mlir.lower_jaxpr_to_module(
+  effects = list(jaxpr.effects)
+  module, _ = mlir.lower_jaxpr_to_module(
       "spjit_{}".format(fun.__name__),
       core.ClosedJaxpr(jaxpr, consts),
+      effects,
       platform=platform,
       axis_context=mlir.ReplicaAxisContext(axis_env),
       name_stack=new_name_stack(wrap_name(name, "sharded_jit")),
